#####R SCRIPT Kestimator version 1.13
#####R SCRIPT written by S.J. Puechmaille
#####Script used and developped for the following paper:
#####Puechmaille SJ (2016) The program STRUCTURE does not reliably recover the correct population structure when sampling is uneven: sub-sampling and new estimators alleviate the problem. Molecular Ecology Resources, 16, 608-627. dx.doi.org/10.1111/1755-0998.12512
#####When using this script, please, cite the above mentioned paper.

#####This script was originally much longer and enabled multiple data sets to be analysed at once (hence it is somehow complex) but to accompany the paper,
#####the script has been simplified and does not allow anymore multiple files to be analysed at once.

##########################################################################
##########################################################################
##############			FILE RENAMING FUNCTION		##############
##########################################################################

#Small function created to easily rename files	
	File_Rename<-function(Folder,motif1,motif2) {
	setwd(Folder)
		#Gets file names
	FileList<-list.files()
		#Searches for what is to be replaced (here 'motif1' to be replaced by 'motif2')
	NewFileList<-gsub(motif1,motif2,FileList)
		#Renames the files when needed
	file.rename(from=FileList,to=NewFileList)
	}


##########################################################################
##########################################################################
##############	FSTAT FILE SUB-SAMPLING FUNCTION		##############
##########################################################################

subsampFSTAT<-function(file_in,path_in,path_out=path_in,ResamplingScheme,ResamplingSubdiv=ResamplingScheme/ResamplingScheme,STRUCTURE=TRUE) {

if	(identical(ResamplingScheme/ResamplingSubdiv,round(ResamplingScheme/ResamplingSubdiv,0))=="FALSE") stop("Number of \"ResamplingScheme\" are not multiple of \"ResamplingSubdiv\"")

	#Reads the data from a data.frame
	#Create the file names 

PathFile_in<-paste(path_in,file_in,".dat",sep="")

	#Calculates the number of loci
Info<-read.table(PathFile_in,nrows=1)
Nlocus<-as.numeric(Info[2])

	#Reads the .dat file out of EASYPOP
sub<-read.table(PathFile_in,skip=(Nlocus+1),colClasses = c("numeric",rep("character",Nlocus)))

	#Calculates the dimensions of the file
dime<-dim(sub)

	#Calculates the number of population and then individuls (NOT assuming the same number of ind per pop)
npop<-length(unique(sub[,1]))
	#Calculates the number of individuals per population (assuming increasing number in the populations names)
IndPerPop<-table(sub$V1)

	#Checks that there are enough individuals to be sampled from
if(min(IndPerPop-ResamplingScheme)<0) stop("Some number of \"ResamplingScheme\" are larger than the original subpopulation sizes")

	#Creates a vector to place indiv numbers from 1 to nindiv
indM<-1:dime[1]

	#Select the new sampling regime
popv<-ResamplingScheme;sum(popv)

	#Creates a matrix to place the results of the resampling
keepM<-rep(0,dime[1])

	#Proceeds with the resampling
for (i in 1:npop)	{
	if (length(which(sub$V1==i))==1 & popv[i]==1)	{keep<-which(sub$V1==i); keepM[keep]<-1}
	if (length(which(sub$V1==i))>1) {
		keep<-sample(which(sub$V1==i),popv[i],replace=FALSE)
		keepM[keep]<-1
			}
		}
	#Number of animals kept
sum(keepM)

	#Identifies animals to keep
keepF<-which(keepM==1)
	#Add (Nlocus+1)(number of lines skipped when reading the input file)
keepF1<-keepF+(Nlocus+1)
	#Add 1 to (Nlocus+1): starting lines for the Fstat format
keepF1<-sort(c(keepF1,1:(Nlocus+1)))

	#Reads the full Fstat file 'as is'
PathFile_out<-paste(path_out,file_in,"_Sub.dat",sep="")
sube<-readLines(PathFile_in,warn=FALSE)

	#Writes only the selected lines from the original Fstat file
write.table(sube[keepF1],PathFile_out,col.names=FALSE,row.names=FALSE,sep="\t",quote=FALSE)


#Start of the BLOCK to rename pop names in consecutive numbers
	#Re-read the Fstat file to change the pop names
fstatIn<-read.fstat(PathFile_out); fstatInNew<-fstatIn

	#Creates a vector to place the new Population name (=number)
NewVecPop<-vector(length=sum(popv),mode="numeric")

	#Start the first pop at this value and then proceed through the loop to change the names
NpopStart<-1
for (h in 1: npop)	{
	OldPopName<-unique(unique(sub[,1]))[h]
		if (ResamplingScheme[h] >0) {		#If the pop is excluded, does not consider it
	OldRows<-which(fstatIn$Pop %in%	OldPopName)
	NewPopSampSize<-rep(ResamplingScheme[h]/ResamplingSubdiv[h],ResamplingSubdiv[h])
	NewPopName<-rep(NpopStart:(NpopStart+ResamplingSubdiv[h]-1),NewPopSampSize)
	NewVecPop[OldRows]<-NewPopName
	NpopStart<-NpopStart+ResamplingSubdiv[h]
		}
	}

#Changes the Population names in the new Fstat file and then exports the file
	fstatInNew$Pop<-NewVecPop
	write.fstat(fstatInNew,PathFile_out)
#Also exports a csv file with old and new Population names
	PathFilePopName_out<-paste(path_in,file_in,"PopNames.csv",sep="")
	NewOldPopNames<-cbind(fstatIn$Pop,NewVecPop);colnames(NewOldPopNames)<-c("OldNames","NewNames")
#End of the BLOCK to rename pop names in consecutive numbers


if(STRUCTURE=="TRUE") {
	#Writes only the selected lines from the original Fstat file but in STRUCTURE format
PathFile_out_Str<-paste(path_out,file_in,"_Sub.str",sep="")
fstatIn2<-read.fstat(PathFile_out)
write.struct(fstatIn2,ilab=TRUE,pop=TRUE,fname=PathFile_out_Str)
	}

	#Reports the number of individual sub-sampled (to be used in the 'equal subsampling')
cat("\n","Number of individuals sub-sampled:",sum(popv)," out of ",dime[1],"\n","New number of subpopulations: ",length(unique(NewVecPop)))
	}

##########################################################################
##########################################################################
##############	STRUCTURE RUNS ANALYSES FUNCTION		##############
##########################################################################

#Function to run the estimators of K
	STRUCTURE_ANALYSIS<-function(Nruns,PercTreshGhost=0.5,Thresh=seq(0.5,0.8,by=0.1),InputFileBase,ResFolder,DatasetQualifier,RunDeltaK=TRUE,PlotMemb=FALSE,NewPopName=FALSE,NewPopSizes,OutPutcsv=TRUE,NewPopNameNo=1) {

#By default:
	Ndatasets<-1
	extra<-TRUE
	setwd(ResFolder)

#########################
##	PART 0, creates the summary table
#########################

#Creates an empty template
	Summary<-matrix(NA,Nruns,9)
	colnames(Summary)<-c("Run_No","run_id","burnin","iterations","k","ln_prob_data","mean_llh","var_llh","mean_alpha")

#For each file, gathers the needed info
		for (i in 1:Nruns) { #i being the number of Datasets analysed
			InputFile<-paste(InputFileBase,"_",DatasetQualifier,"_",i,"_f",sep="")
	StrOut<-readLines(InputFile,warn=FALSE)

#STRUCTURE input lines that contains these elements are kept:
	InFile<-"Input File:    "
	Proba<-"Estimated Ln Prob of Data   = "
	Likel<-"Mean value of ln likelihood = "
	VarLikel<-"Variance of ln likelihood   = "
	Alpha<-"Mean value of alpha         = "
	NKs<-" populations assumed"
	Burnin<-" Burn-in period"
	Chain<-" Reps"

InFileL<-grep(InFile,StrOut)
	InFileLName<- unlist(strsplit(StrOut[InFileL], split = "\\s"))[[6]]
ProbaL<-grep(Proba,StrOut)
	ProbaLValue<- unlist(strsplit(StrOut[ProbaL], split = "\\s"))[[9]]
LikelL<-grep(Likel,StrOut)
 	LikelLValue<- unlist(strsplit(StrOut[LikelL], split = "\\s"))[[7]]
VarLikelL<-grep(VarLikel,StrOut)
	VarLikelLValue<- unlist(strsplit(StrOut[VarLikelL], split = "\\s"))[[8]]
NKsL<-grep(NKs,StrOut)
	NKsLValue<- unlist(strsplit(StrOut[NKsL], split = "\\s"))[[4]]
if (as.numeric(NKsLValue)>1) {
AlphaL<-grep(Alpha,StrOut)
	AlphaLValue<- unlist(strsplit(StrOut[AlphaL], split = "\\s"))[[14]]	}
BurninL<-grep(Burnin,StrOut)
	BurninLValue<- unlist(strsplit(StrOut[BurninL], split = "\\s"))[[4]]
ChainL<-grep(Chain,StrOut)
	ChainLValue<- unlist(strsplit(StrOut[ChainL], split = "\\s"))[[4]]

#Retrieved data is stored in the summary file
	Summary[i,1]<-i
	Summary[i,2]<-InFileLName
	Summary[i,3]<-BurninLValue
	Summary[i,4]<-ChainLValue
	Summary[i,5]<-NKsLValue
	Summary[i,6]<-ProbaLValue
	Summary[i,7]<-LikelLValue
	Summary[i,8]<-VarLikelLValue
	if (as.numeric(NKsLValue)>1)
		Summary[i,9]<-AlphaLValue
}
#Exports the summary table
	write.csv(Summary,paste(InputFileBase,"_",DatasetQualifier,"_results_summary.csv",sep=""))

#########################
##	PART 1, prepare data/tables/matrices
#########################
	StartTime<-Sys.time()	#To figure out how long the run lasts

#Sets the data set Type as 'p' and the directory folder as 'ResFolder'
	p<-DatasetQualifier	#Which (above) can be either "Subset" , "Full" or c("Subset","Full")
	setwd(ResFolder)	#sets the Result folder
#Sets the 'RES_PercTresh=x.x_NewPopName=FALSE' as the folder name for any output (Results)
	ResPathOut<-paste(ResFolder,"RES_",DatasetQualifier,"_ThreshGhost=",PercTreshGhost,"_ThreshMM=",paste(range(Thresh),collapse="-"),"_NewPopName=",NewPopName,"-",NewPopNameNo,"/",sep="")
	dir.create(ResPathOut,showWarnings =FALSE)

#Gets the last folder name before the results to include this into the PDF name
	out1<-gregexpr ("[\\/]{1}", ResFolder);lpath<-length(out1[[1]])
	DataSet<-substr(ResFolder,start=out1[[1]][lpath-2]+1, stop=out1[[1]][lpath-1]-1)

#Vector to store the various K values looked at (K value can be duplicated if duplicate runs are carried out)
	PopSizes<-vector()

#First figures out what K values we deal with (and how many 'replicate' for each)
	start<-1
		for (i in 1:Ndatasets) { #i being the number of Datasets analysed
			resS<-read.csv(paste(InputFileBase,"_",p,"_results_summary.csv",sep=""),h=TRUE,skip=0) #Summary file
				NK<-length(resS$k)
			PopSizes[start:(start+NK-1)]<-t(resS$k)
			start<-start+NK
				#Read one file to get the number of individuals
					Qfile<-paste(InputFileBase,"_",p,"_",1,"_q",sep="");QQres<-read.table(Qfile)
					}

#Number of K values considered and which values (and how many times)
	#Number of K values considered (not the values themsleves)
		nK<-length(unique(PopSizes))
	#Unique K values considered and number of replicates run per K value (and data set)
		Kval<-unique(PopSizes)
		nRep<-length(PopSizes)/Ndatasets/nK
	#Number of runs per data set
		NrepAcrossK<-length(PopSizes)/Ndatasets
	#Number of individuals
		Nind<-length(QQres[,1])
	#Number of populations
		if (NewPopName=="TRUE") {if (file.exists("NewPopNames.txt")=="TRUE" ) {InfoPopNames<-read.table("NewPopNames.txt") ; Npop<-length(unique(InfoPopNames[,1])) } else {InfoPopNames<-as.data.frame(rep(1:(length(NewPopSizes)),NewPopSizes))  ; Npop<-length(unique(InfoPopNames[,1]))} }  else {Npop<-length(unique(QQres[,2]))}

		if (NewPopName=="TRUE") write.csv(table(InfoPopNames),paste(ResPathOut,"NewPopSizes.csv",sep=""))
	#Number of threshold considered for the Max and Median values of K
		NThresh<-length(Thresh)

#Creates the matrices to place the results (Number of replicate per row and number of data set per column)
	for (j in 1:nK) {	#j being the number of replicate per value of K
		#Data set number combined with replicate number (nRep*Ndatasets combinations)
			DataSets<-((paste("Set",sort(rep(1:Ndatasets,nRep)),sep="")));DataSets_RepNo<-paste(DataSets,"Rep",1:nRep,sep="")
		#Cluster with appended the name from 1 to nK (number of K value)
			Clust_nK<-paste("Clus",1:Kval[j],sep="")
			Clust_nK0<-paste("Clus",0:Kval[j],sep="")	#from 0 to nK
		#Number of individuals
			Ind_All<-paste("I",1:Nind,sep="")
		assign(paste(InputFileBase,"_","Max_K",Kval[j],sep=""),matrix(0,nRep*Ndatasets,Kval[j],dimnames=list(DataSets_RepNo,Clust_nK)))
		assign(paste(InputFileBase,"_","MaxMedian_K",Kval[j],sep=""),matrix(0,nRep*Ndatasets,Kval[j],dimnames=list(DataSets_RepNo,Clust_nK)))
		assign(paste(InputFileBase,"_","Sd_K",Kval[j],sep=""),matrix(0,nRep*Ndatasets,Kval[j],dimnames=list(DataSets_RepNo,Clust_nK)))
		assign(paste(InputFileBase,"_","SdAll_K",Kval[j],sep=""),matrix(0,nRep*Ndatasets,Kval[j],dimnames=list(DataSets_RepNo,Clust_nK)))
		assign(paste(InputFileBase,"_","Coun0.5_K",Kval[j],sep=""),matrix(0,nRep*Ndatasets,Kval[j],dimnames=list(DataSets_RepNo,Clust_nK)))
		assign(paste(InputFileBase,"_","CountK_K",Kval[j],sep=""),matrix(0,nRep*Ndatasets,Kval[j],dimnames=list(DataSets_RepNo,Clust_nK)))
		assign(paste(InputFileBase,"_","Count_N",Kval[j],sep=""),matrix(0,nRep*Ndatasets,Kval[j]+1,dimnames=list(DataSets_RepNo,Clust_nK0)))
		assign(paste(InputFileBase,"_","MaxAss_K",Kval[j],sep=""),matrix(0,nRep*Ndatasets,Nind,dimnames=list(DataSets_RepNo,Ind_All)))
				}
#Prepares the matrix to place the best value of K (delta K, Posterior probability of K, etc.)
	BestK<-as.data.frame(matrix(0,Ndatasets,nK*3))
	colnames(BestK)<-c(paste("Delta_K=",Kval,sep=""),paste("PP_K=",Kval,sep=""),paste("ln_K=",Kval,sep="")			)

#Generates some headers to be used further down to label columns and rows of objects used to place results
	#Data set number
		DataSetsUnique<-paste("Set",1:Ndatasets,sep="")
	#Data set number combined with replicate number (nRep*Ndatasets combinations)
		DataSets<-((paste("Set",sort(rep(1:Ndatasets,nRep)),sep="")));DataSets_RepNo<-paste(DataSets,"Rep",1:nRep,sep="")
	#Data set number
		DataSetsNo<-((paste("Set",sort(1:Ndatasets),sep="")))
	#Cluster with appended the name from 1 to nK (number of K value)
		Clust_nK<-paste("Clus",1:Kval[j],sep="")
	#K values from 1 to nK in the form 'K=x'
		Kvalues<-paste("K=",1:nK,sep="")
	#Thresholds
		ThreshHead<-paste("Thresh",Thresh,sep="")

#########################
##	PART 2, fills in data/tables/matrices
#########################

	#######################################
	###Start of the loop for each data set#
	#######################################

Added<-1
VAdded<-vector()
		for (i in 1:Ndatasets) {
#Sets to 0 the value of K so that when it starts, KNew is different from KBefore (see Trick below)
	KBefore<-0

#Reads the summary file with lnP(D)
	resS<-read.csv(paste(InputFileBase,"_",p,"_results_summary.csv",sep=""),h=TRUE,skip=0) #Summary file

		########################################
		###Start of the loop for each replicate#
		########################################

#Start saving the membership of indiv as a PDF
	if(PlotMemb==TRUE)	{
	pdf(paste(ResPathOut,"Membership_Dataset-No.",i,"_",DataSet,"_",DatasetQualifier,".pdf",sep=""),width=38,height=20)
	par(mfrow=c(5,1),mai=c(0.5 ,0.82 ,0.48, 0.42))	}	#mai default 1.02 0.82 0.82 0.42

#For each data set goes through each replicate
	for (v in (1):(length(nK)*NrepAcrossK))	{		#####!!!!!!!!!!!!!!!!!!!!!!!!!!!!############## instead of (1), might need (Nrep+1)

#Imports and analyses the 'q' files
	qfile<-paste(InputFileBase,"_",p,"_", v,"_q",sep="");qres<-read.table(qfile)
	qRes<-as.matrix(qres[,-c(1:2)]);dimer<-dim(qRes)

#Generates further headers to be used further down to label columns and rows of objects used to place results
	#Populations numbers as defined by the user prior to data analysis (e.g. sampled colonies/populations)
		if (NewPopName=="TRUE") {Pops<-paste("Pop",1:length(unique(InfoPopNames[,1])),sep="")} else {Pops<-paste("Pop",1:length(unique(qres$V2)),sep="")}
	#Cluster with appended the name from 1 to nK (but nK incremented as needed in the loop [hence different from 'Clust_nK' above])
		Clust_nKInc<-paste("Clus=",1:(dimer[2]),sep="")
	
#Creates 2 matrices to place the summary results of the run (per Pop and for the different cluster and incremented for each K value)
	if (NewPopName=="TRUE") {SummaryMean<-as.data.frame(matrix(0,length(unique(InfoPopNames[,1])),dimer[2],dimnames=list(Pops,Clust_nKInc)))} else {SummaryMean<-as.data.frame(matrix(0,length(unique(qres$V2)),dimer[2],dimnames=list(Pops,Clust_nKInc)))}
	SummaryMedian<-SummaryMean
	SummarySd<-SummaryMean

#VEctors to place the start, end and median of each pop (in terms of no. of indiv) (Used to plot membership coefs below)
	Start<-vector(); End<-vector(); Med<-vector()

#Calculates the average proba of indiv to belong to various Populations (as defined prior to running the analysis)
	for (j in 1:Npop) {		#For each Populations (as defined by the user)
			if (NewPopName=="TRUE") {Uni<-sort(unique(InfoPopNames[,1])); select<-which(InfoPopNames[,1]==Uni[j])} else {Uni<-sort(unique(qres$V2)); select<-which(qres$V2==Uni[j])}
				Start[j]<-min(select);End[j]<-max(select);Med[j]<-median(select)
			SummaryMean[j,]<-apply(as.data.frame(qRes[select,]),2,function(x) mean(x))
			SummaryMedian[j,]<-apply(as.data.frame(qRes[select,]),2,function(x) quantile(x,0.5))
			SummarySd[j,]<-apply(as.data.frame(qRes[select,]),2,function(x) sd(x))
		}

#Trick to separate runs with different values of K and place the resuls below in the right place in the matrices
	KNew<-dimer[2]
	ifelse (KNew == KBefore,rank<-rank+1,rank<-1)
	KBefore<-KNew

#For incrementation
	Increm<-i*nRep-nRep+rank 
	Incr<-i*NrepAcrossK-NrepAcrossK+v

#Places in the matrices the:
	#Max % assignment of any Pop to each clusters (done for eack K value separately) based on the arithmetic mean of Q
				Maxe<-get(paste(InputFileBase,"_","Max_K",PopSizes[Incr],sep=""))	
				Maxe[Increm,]<-apply(SummaryMean,2,function(x) max(x))
				assign(paste(InputFileBase,"_","Max_K",PopSizes[Incr],sep=""),Maxe)
	#Max % assignment of any Pop to each clusters (done for eack K value separately) based on the median of Q
				MaxeMed<-get(paste(InputFileBase,"_","MaxMedian_K",PopSizes[Incr],sep=""))	
				MaxeMed[Increm,]<-apply(SummaryMedian,2,function(x) max(x))
				assign(paste(InputFileBase,"_","MaxMedian_K",PopSizes[Incr],sep=""),MaxeMed)
	#Number of Pops having a mean assignment to the cluster in question >0.5
				Coun0.5<-get(paste(InputFileBase,"_","Coun0.5_K",PopSizes[Incr],sep=""))	
				Coun0.5[Increm,]<-apply(SummaryMean,2,function(x) length(which(x>=0.5)))
				assign(paste(InputFileBase,"_","Coun0.5_K",PopSizes[Incr],sep=""),Coun0.5)
	#Number of Pops having a mean assignment to the cluster in question >PercTreshGhost( 0<PercTreshGhost<1 )
				CountK<-get(paste(InputFileBase,"_","CountK_K",PopSizes[Incr],sep=""))	
				CountK[Increm,]<-apply(SummaryMean,2,function(x) length(which(x>=PercTreshGhost)))
				assign(paste(InputFileBase,"_","CountK_K",PopSizes[Incr],sep=""),CountK)
	#Number of individuals having an assignment to the cluster in question >0.5
				qresS<-qRes[,1:(PopSizes[Incr])]
				CountN<-get(paste(InputFileBase,"_","Count_N",PopSizes[Incr],sep=""))	
				IndMaxAssCol<-apply(as.matrix(qresS),1,which.max)
				IndMaxAssVal<-apply(as.matrix(qresS),1,max)
				Garde<-which(IndMaxAssVal<=0.5)
				IndMaxAssCol[Garde]<-0
				rangeR<-(-0.5):(PopSizes[Incr]+0.5)
				I2C<-hist(IndMaxAssCol,plot=FALSE,breaks=rangeR)
				CountN[Increm,]<-I2C$counts
				assign(paste(InputFileBase,"_","Count_N",PopSizes[Incr],sep=""),CountN)
	#Plots membership coefficient of all indiv.
				if(PlotMemb==TRUE) {
				Nclu<-length(which(Maxe[Increm,]>0.5))
				COLE<-colors()[c(23,26,30,33,36,38,43,47,50,51,53,56,100,450,552,612,652,656)]	#rainbow(20)
				barplot(t(qresS),las=1,cex.main=2.5,cex.axis=2,col = COLE,border = NA ,width=1,xlim=c(1,Nind),space=0,main=paste("K=",PopSizes[Incr],", Corrected K=",Nclu,", dataset ",i,", run ",v,", ln Pr(X|K)=",resS$ln_prob_data[v]," / min(ln Pr(X|K))=",max(resS$ln_prob_data),sep=""))
				abline(v=Start-1);abline(v=End)
		#Get real clusters and numbers to plot
			#For the mean >0.5
				wk1<-(apply(SummaryMean,1,function(x) max(x)));wk2<-wk1;wk2[wk2<0.5]<-0;wk2[wk2>=0.5]<-1
				wk3<-wk2*(apply(SummaryMean,1,which.max))
				wk4<-vector(mode="numeric",length=length(wk3));wk5<-wk3
				vale<-1
				for (d in 1:length(wk5)) {
					Cherch<-wk5[d] 
						if (Cherch>0 ) {
							PopSel<-which(wk5==Cherch)
							wk5[PopSel]<-0
							wk4[PopSel]<-vale
							vale<-vale+1
						}
					}
			#For the median >0.5
				wk11<-(apply(SummaryMedian,1,function(x) max(x)));wk21<-wk11;wk21[wk21<0.5]<-0;wk21[wk21>=0.5]<-1
				wk31<-wk21*(apply(SummaryMean,1,which.max))
				wk41<-vector(mode="numeric",length=length(wk31));wk51<-wk31
				vale<-1
				for (d in 1:length(wk51)) {
					Cherch<-wk51[d] 
						if (Cherch>0 ) {
							PopSel<-which(wk51==Cherch)
							wk51[PopSel]<-0
							wk41[PopSel]<-vale
							vale<-vale+1
						}
					}
			#Adding legend to PLOTS
				text(pos=1,x=Med,y=-0.04,labels=paste(Uni,"-",wk4,".",wk41,sep=""),xpd=TRUE,cex=2,srt=30)	}
					#Fills in the Pop2Clus with the labels printed on the PLOT
	#Max assignment of each individual to any cluster
				MaxAss<-get(paste(InputFileBase,"_","MaxAss_K",PopSizes[Incr],sep=""))	
				MaxAss[Increm,]<-IndMaxAssVal
				assign(paste(InputFileBase,"_","MaxAss_K",PopSizes[Incr],sep=""),MaxAss)
		}

	if(PlotMemb==TRUE) 
		dev.off()


		########################################
		###End of the loop for each replicate###
		########################################

	##################################################
	##	RUNS the Delta K method, Evanno (START)	##
	##################################################

#Only runs if the method has been selected
	if (RunDeltaK=="TRUE") {

#Reads the summary file with lnP(D)
	resS<-read.csv(paste(InputFileBase,"_",p,"_results_summary.csv",sep=""),h=TRUE,skip=0) #Summary file
#Create a matrix to palce the calculations of Delta K (according to Evanno et al. 2005)
	Delta<-as.data.frame(matrix(0,nK,5)); colnames(Delta)<-c("L(K)","SD","L'(K)","L''(K)","Delta K")
		debuT<-seq(1,NrepAcrossK,by=nRep)
			for (u in debuT)	{
				debNumb<-which(u == debuT)
				Delta[debNumb,1]<-mean(resS$ln_prob_data[u:(u+nRep-1)]) # Ln P(D) Average or L(K)
				Delta[debNumb,2]<-sd(resS$ln_prob_data[u:(u+nRep-1)]) # Ln P(D) Standard Deviation (sd)
						}
		#Calculates L'(K)
			for (g in 2:nK) 
				Delta[g,3]<-Delta[g,1]-Delta[(g-1),1]	#L'(K)
		#Calculates L''(K)
			for (g in 2:nK) 
				Delta[g,4]<-abs(Delta[(g+1),3]-Delta[g,3])	#L''(K)
		#Calculates Delta(K)	
			#Adds a small value for SD=0 (otherwise, Delata K is incalculate as dividing by 0) and gives a warning message
				SD0<-which(Delta[,"SD"]==0)
				if (length(SD0)>0) {Delta[SD0,"SD"]<-1e-10;warning(paste("Delta K calculation problem, set ",i,sep="")); write(paste("dataset",i,sep=" "),paste(ResPathOut,"A_DeltaK-SD=0_Warning.txt",sep=""),append=TRUE)}
			Delta[,5]<-Delta[,4]/Delta[,2]
		#Places the results in the global matrix (Best K)
			BestK[i,(1:nK)]<-t(Delta[,5])
		#Places LnP(D) into the BestK matrix
			BestK[i,((2*nK+1):(3*nK))]<-t(Delta[,1])	
	}

##################################################
##   Calculates the Posterior Probability of Ks ##
##################################################

#Calculates the posterior probability of each K value per data set

#Prepare the lnP(D)
	lnE<-as.data.frame(cbind(resS$ln_prob_data,as.factor(resS$k)))
	lnK<-as.matrix(unstack(lnE));colnames(lnK)<-c(seq(1:length(lnK[1,])))

# Standardize by subtracting highest value
	StandLn<- lnK - lnK[which.max(lnK)]
# Standardized means of each K value
	MeanLn = as.vector(apply(StandLn, 2, mean))
# Exponentiate
	MeanLnTrans<-mpfr(MeanLn,200000)
	ExpLn <- as.vector(exp(MeanLnTrans))

# Divide by sum of all model likelihoods
	PostProb<-as.numeric(ExpLn /sum(ExpLn ))

#Places the Posterior probability into the 'BestK' matrix
	BestK[i,(nK+1):(nK*2)]<-c(PostProb)

#######################################################
##   Calculates the Posterior Probability of Ks (END)##
#######################################################

#######################################################
##   Calculates the MeanK, MedianK, etc. 	    (START)##
#######################################################

#Only runs if the 'extras' has been selected
	if (extra=="TRUE") {

	#Creates a data frame to place the summary per data set (number of run with correct K value guessed)
		perSum<-as.data.frame(matrix(nRep,Ndatasets,nK+1)); colnames(perSum)<-c(paste("K=",Kval,sep=""),"Nruns")
	#Creates arrays to place the correct number of Ks per data set (across replicates) for:	
		#The number of clusters with at least 1 pop with average membership coef > values in Thresh	
			KmedianMean<-array(0,dim=c(Ndatasets,(nK+1),NThresh),dimnames=list(NULL,c(paste("K=",Kval,sep=""),"Nruns"),NULL)) ; KmedianMean[,nK+1,]<-Ndatasets
			KmedianMedian<-array(0,dim=c(Ndatasets,(nK+1),NThresh),dimnames=list(NULL,c(paste("K=",Kval,sep=""),"Nruns"),NULL)) ; KmedianMedian[,nK+1,]<-Ndatasets
			KmaxMean<-array(0,dim=c(Ndatasets,(nK+1),NThresh),dimnames=list(NULL,c(paste("K=",Kval,sep=""),"Nruns"),NULL)) ; KmaxMean[,nK+1,]<-Ndatasets
			KmaxMedian<-array(0,dim=c(Ndatasets,(nK+1),NThresh),dimnames=list(NULL,c(paste("K=",Kval,sep=""),"Nruns"),NULL)) ; KmaxMedian[,nK+1,]<-Ndatasets
		#The number of populations that have an average membership coef >0.5 for that cluster		
			KT<-perSum ; KTHigh<-perSum
		#The number of ghost cluster
			GhostSet<-array(0,dim=c(Ndatasets,(nK+1),NThresh),dimnames=list(NULL,c(paste("K=",Kval,sep=""),"Nruns"),NULL))
		#The number of real cluster (median per run) and per K value
			Kreal_Run<-array(0,dim=c(Ndatasets,(nK+1),NThresh),dimnames=list(NULL,c(paste("K=",Kval,sep=""),"Nruns"),NULL))

		l1<-(rep(paste("Set",sort(rep(1:Ndatasets,nRep)),sep="")));RowNames<-paste(l1,"Rep",1:nRep,sep="")
		ColNames<-paste("K=",1:nK,sep="")
		HeadNames<-list(RowNames,ColNames)
	Kpercent<-as.data.frame(matrix(0,nRep*Ndatasets,nK,dimnames=HeadNames))
	Kpercent0<-Kpercent
		#Creates a matrix to place the ghost clusters (for all data sets and all K values)	
			GhostAll<-array(0,dim=c(nRep*Ndatasets,nK,NThresh),dimnames=list(RowNames,ColNames,Thresh))
			GhostAllMedian<-GhostAll

	for (j in 1:nK) {
		#Gets the matrices
			Aver<-get(paste(InputFileBase,"_","Max_K",Kval[j],sep=""))
			AverMed<-get(paste(InputFileBase,"_","MaxMedian_K",Kval[j],sep=""))
			CountK<-get(paste(InputFileBase,"_","CountK_K",Kval[j],sep=""))
			CountN<-get(paste(InputFileBase,"_","Count_N",Kval[j],sep=""))
			MaxAss<-get(paste(InputFileBase,"_","MaxAss_K",Kval[j],sep=""))
	for (zz in 1:NThresh)	{		#does the calculation for each threshold (Thresh) value
		#Calculates some summary to add to the file when exporting
			GhostCluster<-apply(Aver,1,function(x) length(which(x<=Thresh[zz])))
			GhostClusterMedian<-apply(AverMed,1,function(x) length(which(x<=Thresh[zz])))	
		#GhostAll = matrix with the number of ghost cluster per replicate (row) and per value of K (column)
			GhostAll[,j,zz]<-GhostCluster	
			GhostAllMedian[,j,zz]<-GhostClusterMedian	
					}

		#Writes down the results as .csv files
			if (OutPutcsv == TRUE ) {
			write.csv(CountN,file=paste(ResPathOut,"Results_Master_",InputFileBase,"_",p,"_","CountInd_K=",Kval[j],".csv",sep=""))
					}
		#Calculates how many clusters do not have any pop assigned to
			#Does this using multiple thresholds stored in Thresh, hence the loop	
				for (zz in 1:NThresh)	{
					assign(paste("ghostMean_",zz,sep=""),apply(Aver,1,function(x) length(which(x>Thresh[zz]))))
					assign(paste("ghostMedian_",zz,sep=""),apply(AverMed,1,function(x) length(which(x>Thresh[zz]))))
					}
		#Calculates how many pops have an average membership coef >PercTreshGhost for that cluster
			KTresh<-apply(CountK,1,function(x) length(which(x>0)))
		#Calculates the number of replicate where K has been guessed correctly
			debut<-seq(1,nRep*Ndatasets,by=nRep)
				for (u in debut)	{
					debNum<-which(u == debut)		
					perf<-apply(Coun0.5,1,function(x) length(which(sd(x)==0 & mean(x)==1)))
					perSum[debNum,j]<-sum(perf[u:(u+nRep-1)])
						#Does this using multiple thresholds stored in Thresh, hence the loop
						for (zz in 1:NThresh)	{
							#Retrieve the Means and Median for the correct Threshold
								MEAN<-get(paste("ghostMean_",zz,sep=""))
								MED<-get(paste("ghostMedian_",zz,sep=""))
							#Gets the number of population (based on mean assignment of indivs) that is not ghost (in 2 different ways, either median or max)
								KmedianMean[debNum,j,zz]<-ceiling(quantile(MEAN[u:(u+nRep-1)],probs=0.5,type=7))
								KmaxMean[debNum,j,zz]<-max(MEAN[u:(u+nRep-1)])
							#Gets the number of population (based on median assignment of indivs) that is not ghost (in 2 different ways, either median or max)
								KmedianMedian[debNum,j,zz]<-ceiling(quantile(MED[u:(u+nRep-1)],probs=0.5,type=7))
								KmaxMedian[debNum,j,zz]<-max(MED[u:(u+nRep-1)])
										}
						KT[debNum,j]<-ceiling(quantile(KTresh[u:(u+nRep-1)],probs=0.5,type=7))
							}

		#Calculates:
			#Calculates to which cluster the indiv from each pop belong to with a proportion above a treshold (in average)
			#Then if there is a cluster with proportion > treshold, get 1, otherwise, 0
			#Looks for the highest K when all pops belong with proportion > treshold to 1 cluster 
				Kpercent[,j]<-apply(CountK,1,function(x) length(which(x>0)))
				Kpercent0[,j]<-apply(CountK,1,function(x) length(which(x==0)))
		}
			#Writes the ghost matrix created and filled in the loop above
				write.csv(GhostAll,file=paste(ResPathOut,"Results_Master_",InputFileBase,"_",p,"_","Spurious_Mean.csv",sep=""))
				write.csv(GhostAllMedian,file=paste(ResPathOut,"Results_Master_",InputFileBase,"_",p,"_","Spurious_Median.csv",sep=""))
				}
			}

##Correct for ghost clusters for Delta K and PP and LnK
	GhostAllKval<-array(sort(rep(Kval,nRep*Ndatasets)),dim=c(nRep*Ndatasets,nK,NThresh),dimnames=list(RowNames,ColNames,Thresh))
		RealKRep<-GhostAllKval-GhostAll

		#Calculates the number of ghost clusters per data set (and each K values)
		#GhostSet = median number of ghost cluster for each data set (row) and each K value (column)
			debut<-seq(1,nRep*Ndatasets,by=nRep)
				for (u in debut)	{
				debNum<-which(u == debut)
					for (zz in 1:NThresh)	{
						Subs<-GhostAll[u:(u+nRep-1),,zz]							#Subs=matrix of ghost clusters for a single data set for each replicate(row) and each K value (column)
							GhostSet[debNum,(1:nK),zz]<-round(apply(Subs,2,median))		#Calculation of ghost clusters per replicate and then within each data set
						Subs1<-RealKRep[u:(u+nRep-1),,zz]						#Subs=matrix of real clusters for a single data set for each replicate(row) and each K value (column)
							Kreal_Run[debNum,(1:nK),zz]<-round(apply(Subs1,2,median))	#Calculation of real clusters per replicate and then within each data set
									}
							}	

#####################################################################################################################
###End of the loop for each data set#################################################################################
#####################################################################################################################

	write.csv(BestK,file=paste(ResPathOut,"Results_BestK_",InputFileBase,"_",p,".csv",sep=""))

##Some more calculations on Delta K success
	bestKK<-BestK[,1:(nK)]
	DeltaK<-apply(bestKK,1,which.max)
##Some more calculations on Posterior Probability success
	bestKPP<-BestK[,(nK+1):(2*nK)]
	PosProbK<-apply(bestKPP,1,which.max)
##Some more calculations on likelihood success
	bestKLn<-BestK[,(2*nK+1):(3*nK)]
	LnPK<-apply(bestKLn,1,which.max)
##Some more calculations on KT score success (max number of clusters with at least 1 pop belonging to)
	bestTK<-KT[,1:(nK)]
	TiK<-apply(bestTK,1,max)

						for (zz in 1:NThresh)	{
##Some more calculations on Median score success (using the arithmetic mean)
assign(paste("bestMedMea_",zz,sep=""),KmedianMean[,1:(nK),zz])
	if(Ndatasets == 1) assign(paste("MedMeaK_",zz,sep=""),max(KmedianMean[,1:(nK),zz]))
	if(Ndatasets > 1)  assign(paste("MedMeaK_",zz,sep=""),apply(KmedianMean[,1:(nK),zz],1,max))
##Some more calculations on the Max score success (using the arithmetic mean)
assign(paste("bestMaxMea_",zz,sep=""),KmaxMean[,1:(nK),zz])
	if(Ndatasets == 1)  	assign(paste("MaxMeaK_",zz,sep=""),max(KmaxMean[,1:(nK),zz]))
	if(Ndatasets > 1)  	assign(paste("MaxMeaK_",zz,sep=""),apply(KmaxMean[,1:(nK),zz],1,max))
##Some more calculations on Median score success (using the median)
assign(paste("bestMedMed_",zz,sep=""),KmedianMedian[,1:(nK),zz])
	if(Ndatasets == 1)  	assign(paste("MedMedK_",zz,sep=""),max(KmedianMedian[,1:(nK),zz]))
	if(Ndatasets > 1)  	assign(paste("MedMedK_",zz,sep=""),apply(KmedianMedian[,1:(nK),zz],1,max))
##Some more calculations on the Max score success (using the median)
assign(paste("bestMaxMed_",zz,sep=""),KmaxMedian[,1:(nK),zz])
	if(Ndatasets == 1)  	assign(paste("MaxMedK_",zz,sep=""),max(KmaxMedian[,1:(nK),zz]))
	if(Ndatasets > 1)  	assign(paste("MaxMedK_",zz,sep=""),apply(KmaxMedian[,1:(nK),zz],1,max))
				}

#Exports the calculations just above
	Export<-as.data.frame(matrix(0,(3+NThresh*7),nK,dimnames=list(c("DeltaK","PPK","LnK",paste(sort(rep(c("DeltaKCor","PPKCor","LnKCor"),NThresh)),Thresh,sep=""),paste(sort(rep(c("MedMeaK","MaxMeaK","MedMedK","MaxMedK"),NThresh)),Thresh,sep="")),c(paste("K=",1:nK,sep="")))))
ranger<-(0:nK)+0.5
DK<-hist(DeltaK,plot=FALSE,breaks=ranger)
PK<-hist(PosProbK,plot=FALSE,breaks=ranger)
LK<-hist(LnPK,plot=FALSE,breaks=ranger)

			for (zz in 1:NThresh)	{
				#Retrieves the data and then places it in the correct place in the EXPORT df (for Median of Mean)
					MK5<-hist(get(paste("MedMeaK_",zz,sep="")),plot=FALSE,breaks=ranger)
					Export[paste("MedMeaK",Thresh[zz],sep=""),]<-MK5$counts  ; remove(MK5)
				#Retrieves the data and then places it in the correct place in the EXPORT df (for Max of Mean)
					MK6<-hist(get(paste("MaxMeaK_",zz,sep="")),plot=FALSE,breaks=ranger)
					Export[paste("MaxMeaK",Thresh[zz],sep=""),]<-MK6$counts  ; remove(MK6)				
				#Retrieves the data and then places it in the correct place in the EXPORT df (for Median of Median)
					MK7<-hist(get(paste("MedMedK_",zz,sep="")),plot=FALSE,breaks=ranger)
					Export[paste("MedMedK",Thresh[zz],sep=""),]<-MK7$counts  ; remove(MK7)
				#Retrieves the data and then places it in the correct place in the EXPORT df (for Max of Median)
					MK8<-hist(get(paste("MaxMedK_",zz,sep="")),plot=FALSE,breaks=ranger)
					Export[paste("MaxMedK",Thresh[zz],sep=""),]<-MK8$counts  ; remove(MK8)
						}
	Export["DeltaK",]<-DK$counts
	Export["PPK",]<-PK$counts
	Export["LnK",]<-LK$counts

##Correct for ghost clusters for Delta K and PP and LnK (corrected per replicate)
	for (vt in 1:Ndatasets) {
			for (zz in 1:NThresh)	{
		RefNumDK<-DeltaK[vt]; RefNumPP<-PosProbK[vt] ; RefNumLnPK<-LnPK[vt]
			if (Kreal_Run[vt,RefNumDK,zz] >0) 
		Export[paste("DeltaKCor",Thresh[zz],sep=""),Kreal_Run[vt,RefNumDK,zz]]<-Export[paste("DeltaKCor",Thresh[zz],sep=""),Kreal_Run[vt,RefNumDK,zz]]+1
			if (Kreal_Run[vt,RefNumPP,zz] >0) 
		Export[paste("PPKCor",Thresh[zz],sep=""),Kreal_Run[vt,RefNumPP,zz]]	<-Export[paste("PPKCor",Thresh[zz],sep=""),Kreal_Run[vt,RefNumPP,zz]]+1
			if (Kreal_Run[vt,RefNumLnPK,zz] >0)
		Export[paste("LnKCor",Thresh[zz],sep=""),Kreal_Run[vt,RefNumLnPK,zz]]<-Export[paste("LnKCor",Thresh[zz],sep=""),Kreal_Run[vt,RefNumLnPK,zz]]+1
							}
					}

#Calculates how many have K=0 (where all detected clusters are 'ghost' clusters as defined based o nthe Threshold
	K_0<-Ndatasets-apply(Export,1,sum)
#Then exports all the data
	write.csv(cbind(K_0,Export),file=paste(ResPathOut,"Results_Estimators_",InputFileBase,"_",p,".csv",sep=""))

##################################################
##	PLOTS the Delta K method, Evanno (START)	##
##################################################
#Saves as a PDF

	pdf(paste(ResPathOut,"Results_BestK_ThreshGhost=",PercTreshGhost,"_ThreshMM=",paste(range(Thresh),collapse="-"),"_",InputFileBase,"_",p,".pdf",sep=""),width=16)
	par(mfrow=c(1,2))
#Only runs if the Delta K method has been selected
	if (RunDeltaK=="TRUE") {
	par(mfrow=c(1,3))
#Plots the results of Delta K and PP
	#Delta K, gets the Delta K values	
		DK<-as.matrix(BestK[,(2:(nK-1))])
	#Gets the max value of Delta K per data set (for plotting)
		maxDK<-apply(DK,1,max)
	#Gets the value of K for the Max of Delta K (for plotting)
		maxDKX<-apply(DK,1,which.max)+1
#Gets the positions to be removed (value of K for which Delta K was not calculated)
	XaxOut<-c(1,max(Kval))
#Plots Delta K (actually, it's log) for the first data set
		plot(Kval[-XaxOut],log(t(BestK[1,(2:(nK-1))])),xlab="K values",ylab="Log Delta K",pch=16,ylim=c(min(log(BestK[,(2:(nK-1))]+0.1)),max(log(BestK[,(2:(nK-1))]))),main="DeltaK estimates")
			for (i in 1: Ndatasets) {
				points(Kval[-XaxOut],log(DK[i,]),pch=16) #for all remaining data sets
					}
#Plots in red the highest Delta K per data set
	points(maxDKX,log(maxDK),pch=16,cex=1,col="red")
#Creates a count of K values with the max Delta K
	ctDK<-hist(maxDKX,breaks=c(Kval[-max(Kval)])+0.1,plot=FALSE)
#Adds the number of K-values that got the highest PP across all data sets
	text(Kval[-1],max(log(DK)+0.2),ctDK$counts,cex=0.6)
	}

##################################################
##	PLOTS the Delta K method, Evanno (END)	##
##################################################
##################################################
##		PLOTS the Ln P(D)	 (START)		##
##################################################
#Plots the results of LnP
	#Gets the LnP values	
		LnP<-as.matrix(BestK[,(nK*2+1):(nK*3)])
	#Gets the max value of LnP per data set (for plotting)
		maxLnP<-apply(LnP,1,max)
	#Gets the value of K for the Max of LnP (for plotting)
		maxLnPX<-apply(LnP,1,which.max)
#Plots PP for the first data set
		plot(Kval,t(LnP[1,]),xlab="K values",ylab="Run Likelihood",pch=16,ylim=c(min(LnP),max(LnP)),main="ln Pr(X|K), likelihood of the data given K")
			for (i in 1: Ndatasets) {
				points(Kval,LnP[i,],pch=16) #for all remaining data sets
					}
#Plots in red the highest Delta K per data set
	points(maxLnPX,maxLnP,pch=16,cex=1,col="red")
#Creates a count of K values with the max PP
	ct<-hist(maxLnPX,breaks=c(0,Kval)+0.1,plot=FALSE)
#Adds the number of K-values that got the highest PP across all data sets
	text(Kval,max(maxLnP+150),ct$counts,cex=0.6)

##################################################
##		PLOTS the Ln P(D)	 (END)		##
##################################################
##################################################
## PLOTS the Delta Posterior Probability (START)##
##################################################
#Plots the results of PP
	#Gets the PP values	
		PP<-as.matrix(BestK[,(nK+1):(nK*2)])
	#Gets the max value of PP per data set (for plotting)
		maxPP<-apply(PP,1,max)
	#Gets the value of K for the Max of PP (for plotting)
		maxPPX<-apply(PP,1,which.max)
#Plots PP for the first data set
		plot(Kval,t(PP[1,]),xlab="K values",ylab="Posterior Probability",pch=16,ylim=c(min(PP),max(PP)),main="Posterior Probability estimates")
			for (i in 1: Ndatasets) {
				points(Kval,PP[i,],pch=16) #for all remaining data sets
					}
#Plots in red the highest Delta K per data set
	points(maxPPX,maxPP,pch=16,cex=1,col="red")
#Creates a count of K values with the max PP
	ct<-hist(maxPPX,breaks=c(0,Kval)+0.1,plot=FALSE)
#Adds the number of K-values that got the highest PP across all data sets
	text(Kval,1.02,ct$counts,cex=0.6)


##################################################
## PLOTS the Delta Posterior Probability (END)  ##
##################################################

##################################################
## 	PLOTS the % belonging to cluster (START) 	##
##################################################

if (extra=="TRUE") {
	par(mfrow=c(NThresh,2),mai=c(0.26, 0.62 ,0.42 ,0.12))
		for (zz in 1:NThresh)	{
				DTK<-cbind(Ndatasets-apply(Export[paste("DeltaKCor",Thresh[zz],sep=""),],1,sum),Export[paste("DeltaKCor",Thresh[zz],sep=""),])
				PPK<-cbind(Ndatasets-apply(Export[paste("PPKCor",Thresh[zz],sep=""),],1,sum),Export[paste("PPKCor",Thresh[zz],sep=""),])
			plot(0:(length(DTK)-1),as.numeric(DTK),pch=16,type="b",las=1,ylim=c(0,Ndatasets),xlab="Best K",ylab="No. of data set",main=paste("K (DeltaKCor) (Tresh.=",Thresh[zz],")",sep=""))
			plot(0:(length(PPK)-1),as.numeric(PPK),pch=16,type="b",las=1,ylim=c(0,Ndatasets),xlab="Best K",ylab="No. of data set",main=paste("K (PPKCor) (Tresh.=",Thresh[zz],")",sep=""))
						}

	par(mfrow=c(NThresh,4),mai=c(0.26, 0.62 ,0.42 ,0.12))
		for (zz in 1:NThresh)	{
			hist(get(paste("MedMeaK_",zz,sep="")),breaks=c(-0.5:max(Kval+0.5)),las=1,freq=TRUE,ylim=c(0,Ndatasets),xlab="Best K",ylab="No. of data set",main=paste("K (Median/mean) (Tresh.=",Thresh[zz],")",sep=""))
			hist(get(paste("MaxMeaK_",zz,sep="")),breaks=c(-0.5:max(Kval+0.5)),las=1,freq=TRUE,ylim=c(0,Ndatasets),xlab="Best K",ylab="No. of data set",main=paste("K (Max/mean) (Tresh.=",Thresh[zz],")",sep=""))
			hist(get(paste("MedMedK_",zz,sep="")),breaks=c(-0.5:max(Kval+0.5)),las=1,freq=TRUE,ylim=c(0,Ndatasets),xlab="Best K",ylab="No. of data set",main=paste("K (Median/median) (Tresh.=",Thresh[zz],")",sep=""))
			hist(get(paste("MaxMedK_",zz,sep="")),breaks=c(-0.5:max(Kval+0.5)),las=1,freq=TRUE,ylim=c(0,Ndatasets),xlab="Best K",ylab="No. of data set",main=paste("K (Max/median) (Tresh.=",Thresh[zz],")",sep=""))
						}
			}

	##################################################
	## 	PLOTS the % belonging to cluster (END) 	##
	##################################################

dev.off()
Sys.time()-StartTime


##################################################
## 	PLOTS the % belonging to cluster (END) 	##
##################################################
	}



	

##############################################################
##############################################################
#######   END OF THE STRUCTURE_ANALYSIS FUNCTION	########
##############################################################
##############################################################











